/******************************************************************************
*
* project name:    Bookworm
* file name:       game##.h
* initial date:    3.07.2005
* author:          mhubi@freesurf.ch
* description:     all "game"-specific functions
*
*******************************************************************************
*
* Bookworm -- another snake game
* Copyright (C) 2005  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h10.h"


volatile int counter = 0;
BOOL option, cursor;
BOOL OptionsOpened;


DEFINE_INT_HANDLER(CursorBlink)
{
  counter++;
  
  if (counter > 9) {
  	cursor = !cursor;
  	counter = 0;
  	GrayDrawChar(CALC(23,63), CALC(76+(option*8),90+(option*8)), 17, A_XOR);
  	GrayDrawChar(CALC(131,171), CALC(76+(option*8),90+(option*8)), 18, A_XOR);
  }
}



void ChooseOptions(void) {
	
	int key;
	char string[4];
	
	option = 0;
	cursor = FALSE;
	OptionsOpened = TRUE;
	
	FontSetSys(F_6x8);
	SetIntVec(AUTO_INT_5, CursorBlink);
	
	do {
		
		while (!(key = KeyPressed())) {
			pokeIO(0x600005, 0b10111);
		}
		
		if (key == K_RIGHT) {
			if (!option) {
				game->level = ((game->level+1) % 11) + ((game->level+1) / 11);
				sprintf(string, " %d", game->level);
			}
			else {
				game->speed = ((game->speed+1) % 11) + ((game->speed+1) / 11);
				sprintf(string, " %d", game->speed);			
			}
			GrayDrawStrExt(CALC(116,156)-((strlen(string)/3)*6), CALC(76+(option*8),90+(option*8)), string, A_REPLACE, F_6x8);
		}
		
		if (key == K_LEFT) {
			if (!option) {
				game->level = game->level-1 ? game->level-1 : 10; 
				sprintf(string, " %d", game->level);
			}
			else {
				game->speed = game->speed-1 ? game->speed-1 : 10;
				sprintf(string, " %d", game->speed);			
			}
			GrayDrawStrExt(CALC(116,156)-((strlen(string)/3)*6), CALC(76+(option*8),90+(option*8)), string, A_REPLACE, F_6x8);
		}
		
		if ((key == K_UP && option) || (key == K_DOWN && !option)) {							
			if (cursor) {
				while (counter)	continue;
			}
			
			option = !option;
		}
		
		Wait();
		
	} while (key != K_ENTER);
	
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);
	
	OptionsOpened = FALSE;
	
}


void GameLoop(void) {
	
	game->score = 0;
	
	do {
		
		InitRound();
		
		Round();
		
		if (game->status != NEWGAME) {
			
			DrawMessage(CONGRAT);
			Waitkey();
			DrawMessage(SCORE);
			Waitkey();
			
			game->level++;
			
			if (game->level == 11) {
				DrawMessage(SPEEDLEVEL);
				Waitkey();
				DrawMessage(NEXTSPEED);
				Waitkey();
				EasterEgg();
			}
			
			DrawMessage(NEXTLEVEL);
			Waitkey();
		
		}
		
	} while (game->status == NORMAL);
	
}

void InitRound(void) {
	
	unsigned int loop1, loop2;
	unsigned int x, y;
	
	game->eaten = 0;
	
	pmark->number = 0;
	
	worm->limbs = 2;
	worm->limbs_left = 0;
	worm->power = 150;
	worm->direction = RIGHT;
	
	worm->pos[0][0] = 11; worm->pos[0][1] = 7;
	worm->pos[1][0] = 10; worm->pos[1][1] = 7;
	
	for (loop1 = 0; loop1 < 24; loop1++) {
		game->field[loop1][0] = WALL;
		game->field[loop1][15] = WALL;
	}
	for (loop1 = 1; loop1 < 15; loop1++) {
		game->field[0][loop1] = WALL;
		game->field[23][loop1] = WALL;
	}
	
	for (loop1 = 1; loop1 < 15; loop1++) {
		for (loop2 = 1; loop2 < 23; loop2++) {
			game->field[loop2][loop1] = EMPTY;
		}
	}
	
	game->field[11][7] = BODY;
	game->field[10][7] = BODY;
	
	for (loop1 = 1; loop1 <= 26; loop1++) {
		
		do {
			x = random(22) + 1;
			y = random(14) + 1;
			
			if (game->field[x][y] == EMPTY) {
				game->field[x][y] = loop1;
				x = 0;
			}
		} while (x != 0);
		
	}
	
}

void Round(void) {
	
	int key, key_temp;
	unsigned int loop1;
	int x, y;
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(0,40), CALC(0,14), CALC(159,199), CALC(99,113), COLOR_WHITE);
	
	DrawField();
	
	DrawWorm();
	
	DrawMessage(START);
	
	while ((key = KeyPressed() != K_ENTER)) {
		pokeIO(0x600005, 0b10111);
	}
	
	DrawMessage(LS);
	
	do {
		
		key_temp = 0;
		key = 0;
		
		for (loop1 = 0; loop1 < 60 - game->speed*4; loop1++) {
			
			/*key_temp = KeyPressed();
			Delay(1, 1);
			
			if (key_temp >= K_UP && key_temp <= K_LEFT && key == 0) {
				
				key = key_temp;
				
			}*/
			
			Delay(1,1);
			if (_keytest(RR_LEFT)) key_temp = K_LEFT;
			else if (_keytest(RR_RIGHT)) key_temp = K_RIGHT;
			else if (_keytest(RR_UP)) key_temp = K_UP;
			else if (_keytest(RR_DOWN)) key_temp = K_DOWN; 
			
			else if (_keytest(RR_F2)) {
				LeadOut(FASTQUIT);
			}
			else if (_keytest(RR_ESC)) {
				Menu();	
			}
			
			if (key_temp >= K_UP && key_temp <= K_LEFT && key == 0) {
				key = key_temp;
			}
			
		}
		
		
		if (key == worm->direction) {
			GameOver();
			return;
		}
		
		if (key == K_UP) {
			worm->direction = UP;
		}
		else if (key == K_DOWN) {
			worm->direction = DOWN;
		}
		else if (key == K_RIGHT) {
			worm->direction = RIGHT;
		}
		else if (key == K_LEFT) {
			worm->direction = LEFT;
		}
		
		//crash-check
		
		x = worm->pos[0][0];
		y = worm->pos[0][1];
		
		if (worm->direction == UP) {
			y--;
		}
		else if (worm->direction == DOWN) {
			y++;
		}
		else if (worm->direction == RIGHT) {
			x++;
		}
		else if (worm->direction == LEFT) {
			x--;
		}
		
		worm->power--;
		DrawPower(worm->power);
		
		if (worm->limbs_left == 0) {
			game->field[worm->pos[worm->limbs-1][0]][worm->pos[worm->limbs-1][1]] = EMPTY;
		}
		
		if (game->field[x][y] == WALL) {
			GameOver();
			return;
		}
		else if (game->field[x][y] == BODY) {
			GameOver();	
			return;
		}
		else if (game->field[x][y] > 0) {
			if (game->field[x][y] == game->eaten + 1) {
				
				game->eaten++;
				worm->limbs_left += game->level;
				
				IncreasePower(10);
				worm->power += 10;
				
				if (game->field[x][y] == 26) {
					return;
				}
				
				game->field[x][y] = BODY;
				
			}
			else if (game->field[x][y] == POWER) {
				IncreasePower(20);
				worm->power += 20;
				
				pmark->number--;
				game->field[x][y] = BODY;
				
				for (loop1 = 0; loop1 < 5; loop1++) {	
					if (pmark->pos[loop1][0] == x && pmark->pos[loop1][1] == y) {
						break;
					}
				}
				for (; loop1 < 4; loop1++) {
					pmark->pos[loop1][0] = pmark->pos[loop1+1][0];
					pmark->pos[loop1][1] = pmark->pos[loop1+1][1];
				}
				
			}
			else {
				GameOver();
				return;
			}
			GraySprite8_AND_R(CALC(-2,38) + x*6, CALC(-2,12) + y*6, 6, mask, mask, LPLANE, DPLANE);
		}
		
		if (worm->limbs_left > 0) {
			worm->limbs++;
			worm->pos[worm->limbs-1][0] = worm->pos[worm->limbs-2][0];
			worm->pos[worm->limbs-1][1] = worm->pos[worm->limbs-2][1];
			worm->limbs_left--;
		}
		
		MoveWorm();
		
		worm->pos[0][0] = x;
		worm->pos[0][1] = y;
		
		DrawWorm();
		
		//worm->power--;
		if (worm->power > 150) {
			worm->power = 150;
		}
		//DrawPower();
		else if (worm->power == 0) {
			GameOver();
			return;
		}
		
		if (random(80) == 39 && pmark->number < 5) {
			PlacePmark();
		}
		BlinkPmark();
		
		
	} while (game->status == NORMAL);
	
	Waitkey();
	LeadOut(QUIT);
	
}

void GameOver(void) {
	
	int key = 0;
	
	DrawMessage(CRASH);
	
	Waitkey();
	
	DrawMessage(SCORE);
	
	Waitkey();
	
	DrawMessage(GAMEOVER);
	
	Waitkey();
	
	CalcHiscores(game->score);
	
	DrawMessage(NEWTRY);
	
	while ((key = KeyPressed()) < K_YES) {
		pokeIO(0x600005, 0b10111);
	}
	
	if (key == K_YES) {
		game->status = NEWGAME;
		GrayClearScreen();
		return;
	}
	
	LeadOut(QUIT);
	
}

void PlacePmark(void) {
	
	int x, y;
	
	do {
		
		x = random(22) + 1;
		y = random(14) + 1;
		
	} while (game->field[x][y] != EMPTY);
	
	
	
	game->field[x][y] = POWER;
	pmark->number++;

	pmark->pos[pmark->number-1][0] = x;
	pmark->pos[pmark->number-1][1] = y;

	
	Sprite8_OR_R(CALC(-2,38) + x*6, CALC(-2,12) + y*6, 6, font[0], LPLANE);
	
}

void EasterEgg(void) {
	
	game->status = QUIT;
	
	GrayClearScreen();
	
	GraySpriteX8_OR(CALC(35,75), CALC(14,22), 66, egg_light, egg_dark, 11, LPLANE, DPLANE);
	
	GrayDrawStrExt(0, CALC(2,10), "FATAL ERROR OCCURED", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
	GrayDrawStrExt(0, CALC(82,90), "LEVEL OVERFLOW", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
	GrayDrawStrExt(0, CALC(93,101), "visit http://boolsoft.mobifiles.de", A_NORMAL | A_CENTERED, F_4x6);
	
	Waitkey();
	
	LeadOut(QUIT);
	
}
