/******************************************************************************
*
* project name:    Bookworm
* file name:       gfx##.h
* initial date:    3.07.2005
* author:          mhubi@freesurf.ch
* description:     all to do with graphics
*
*******************************************************************************
*
* Bookworm -- another snake game
* Copyright (C) 2005  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h10.h"


void DrawWelcome(void) {
	
	unsigned int loop;
	char string[24];
	
	#if defined(TI_BIG)
	DrawFrame();
	#endif
	
	GrayDrawStrExt(CALC(92,132), CALC(15,29), "boolsoft presents", A_NORMAL, F_4x6);
	GrayDrawStrExt(CALC(90,130), CALC(30,44), "bookworm", A_NORMAL | A_SHADOWED, F_8x10);
	GrayDrawStrExt(CALC(104,144), CALC(42,56), COMMENT_VERSION_STRING, A_NORMAL, F_6x8);
	
	sprintf(string, "Level (1-10):  %d", game->level);
	GrayDrawStrExt(CALC(0,0), CALC(76,90), string, A_NORMAL | A_CENTERED, F_6x8);
	sprintf(string, "Speed (1-10):  %d", game->speed);
	GrayDrawStrExt(CALC(0,0), CALC(84,98), string, A_NORMAL | A_CENTERED, F_6x8);
	GrayDrawStrExt(0, CALC(93,107), "Press ENTER to start!", A_NORMAL | A_CENTERED, F_4x6);
	
	//64x66
	GraySpriteX8_OR(CALC(4,44), CALC(4,18), 66, logo_light, logo_dark, 8, LPLANE, DPLANE);
	
	for (loop = 0; loop < 9; loop++) {
		
		GraySprite8_MASK_R(CALC(25,65) + loop*6, CALC(40, 54), 6, limb_light, limb_dark, limb_mask, limb_mask, LPLANE, DPLANE);
		
	}
	
	for (loop = 0; loop < 3; loop++) {
		
		GraySprite8_OR_R(CALC(73,113), CALC(46,60) + loop*6, 6, limb_light, limb_dark, LPLANE, DPLANE);
		
	}
	
	for (loop = 0; loop < 6; loop++) {
		
		GraySprite8_OR_R(CALC(79,119) + loop*6, CALC(58,72), 6, limb_light, limb_dark, LPLANE, DPLANE);
		
	}
	
	GraySprite8_OR_R(CALC(115,155), CALC(58,72), 6, head_light, head_dark, LPLANE, DPLANE);
	
}

void DrawFrame(void) {
	
	unsigned int loopx, loopy;
	
	for (loopx = 0; loopx < 30; loopx++) {
		
		for (loopy = 0; loopy < 16; loopy++) {
			
			GraySprite8_OR_R(1 + loopx*8, 1 + loopy*8, 6, limb_light, limb_dark, LPLANE, DPLANE);
			
		}
		
	}
	
	GrayFastFillRect_R(LPLANE, DPLANE, 40, 14, 199, 113, COLOR_WHITE);
	
}

void DrawField(void) {
	
	unsigned int loopx, loopy, character;
	
	GrayFastOutlineRect_R(LPLANE, DPLANE, CALC(1,41), CALC(1,15), CALC(138,178), CALC(90,104), COLOR_LIGHTGRAY);
	GrayFastOutlineRect_R(LPLANE, DPLANE, CALC(2,42), CALC(2,16), CALC(137,177), CALC(89,103), COLOR_DARKGRAY);
	GrayFastOutlineRect_R(LPLANE, DPLANE, CALC(3,43), CALC(3,17), CALC(136,176), CALC(88,102), COLOR_BLACK);
	
	GrayFastOutlineRect_R(LPLANE, DPLANE, CALC(142,182), CALC(24,38), CALC(153,193), CALC(75,89), COLOR_BLACK);
	DrawPower(worm->power);
	
	for (loopy = 1; loopy <= 14; loopy++) {
		for (loopx = 1; loopx <= 22; loopx++) {
			
			if (game->field[loopx][loopy] > 0) {
				character = game->field[loopx][loopy];
				GraySprite8_OR_R(CALC(4,44) + (loopx-1)*6, CALC(4,18) + (loopy-1)*6, 6, font[character], font[character], LPLANE, DPLANE);
			}
			
		}
	}
	
}

void DrawMessage(int mode) {
	
	char string[24];
	unsigned int lenght, loop;
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(0,40), CALC(91,105), CALC(159,199), CALC(99,113), COLOR_WHITE);
	
	if (mode == START) {
		GrayDrawStrExt2B(0, CALC(92,106), "ENTER to start", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	}
	else if (mode == LS) {
	 sprintf(string, "Level: %d   Speed: %d", game->level, game->speed);
	 GrayDrawStrExt2B(0, CALC(92,106), string, A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
	}
	else if (mode == SCORE) {
		
		game->score += game->eaten*game->level*game->speed*(game->eaten + worm->power/50);
		
		sprintf(string, "Your score: %lu", game->score);
		
		lenght = strlen(string);
		FontSetSys(F_6x8);
		
		GraySprite8_OR_R(CALC(5,45), CALC(93,107), 6, limb_light, limb_dark, LPLANE, DPLANE);
		GraySprite8_OR_R(CALC(5,45) + 6, CALC(93,107), 6, head_light, head_dark, LPLANE, DPLANE);
		Delay(50, 0);
		
		for (loop = 0; loop < lenght + 2; loop++) {
			
			GraySprite8_AND_R(CALC(5,45) + loop*6, CALC(93,107), 6, mask, mask, LPLANE, DPLANE);
			GraySprite8_MASK_R(CALC(5,45) + (loop+1)*6, CALC(93,107), 6, limb_light, limb_dark, limb_mask, limb_mask, LPLANE, DPLANE);
			GraySprite8_OR_R(CALC(5,45) + (loop+2)*6, CALC(93,107), 6, head_light, head_dark, LPLANE, DPLANE);
			
			if (loop < lenght) {
				GrayDrawChar2B(CALC(5,45) + loop*6, CALC(92,106), string[loop], A_NORMAL, DPLANE, DPLANE);
			}
			
			Delay(50, 0);
			
			if (_keytest(RR_F2)) {
				LeadOut(FASTQUIT);
			}
			
		}
		
		
	}
	else if (mode >= CRASH) {
		GrayDrawStrExt2B(0, CALC(92,106), msgtexts[mode], A_NORMAL | A_CENTERED | A_SHADOWED, F_6x8, LPLANE, DPLANE);
	}
	
	
}


void DrawPower(int power) {
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(143,183), CALC(25,39), CALC(152,192), CALC(74,88), COLOR_WHITE);
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(143,183), CALC(25 + (150 - power)/3,39 + (150 - power)/3), CALC(152,192), CALC(74,88), COLOR_DARKGRAY);
	
}


void DrawWorm(void) {
	
	unsigned int loop;
	
	GraySprite8_OR_R(CALC(-2,38) + worm->pos[0][0]*6, CALC(-2,12) + worm->pos[0][1]*6, 6, head_light, head_dark, LPLANE, DPLANE);
	game->field[worm->pos[0][0]][worm->pos[0][1]] = BODY;
	
	for (loop = 1; loop < worm->limbs; loop++) {
		
		GraySprite8_OR_R(CALC(-2,38) + worm->pos[loop][0]*6, CALC(-2,12) + worm->pos[loop][1]*6, 6, limb_light, limb_dark, LPLANE, DPLANE);
		game->field[worm->pos[loop][0]][worm->pos[loop][1]] = BODY;
		
	}
	
}

void MoveWorm(void) {
	
	int loop1;
	
	GraySprite8_AND_R(CALC(-2,38) + worm->pos[0][0]*6, CALC(-2,12) + worm->pos[0][1]*6, 6, mask, mask, LPLANE, DPLANE);
	GraySprite8_AND_R(CALC(-2,38) + worm->pos[worm->limbs-1][0]*6, CALC(-2,12) + worm->pos[worm->limbs-1][1]*6, 6, mask, mask, LPLANE, DPLANE);
	
	for (loop1 = worm->limbs - 2; loop1 >= 0; loop1--) {
		worm->pos[loop1+1][0] = worm->pos[loop1][0];
		worm->pos[loop1+1][1] = worm->pos[loop1][1];
	}
	
}


void IncreasePower(unsigned int value) {
	
	unsigned int value_loop;
	
	for (value_loop = worm->power; value_loop <= worm->power + value; value_loop++) {
		
		if (value_loop > 150) {
			DrawPower(150);
		}
		else {
			DrawPower(value_loop);
		}
		Delay(4, 0);
	} 
	
}

void BlinkPmark(void) {
	
	unsigned int loop;
	
	for (loop = 0; loop < pmark->number; loop++) {
		
		Sprite8_XOR_R(CALC(-2,38) + pmark->pos[loop][0]*6, CALC(-2,12) + pmark->pos[loop][1]*6, 6, font[0], DPLANE);
		
	}
	
}

void DrawMenuFrame(void) {
	
	unsigned int loop;
	
	SpriteX8Get_R(CALC(0,40), CALC(0,14), 100, LPLANE, plane_light, 20);
	SpriteX8Get_R(CALC(0,40), CALC(0,14), 100, DPLANE, plane_dark, 20);
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(7,47), CALC(8,22), CALC(152,192), CALC(93,107), COLOR_WHITE);
	
	for (loop = 0; loop < 24; loop++) {
		GraySprite8_OR_R(CALC(8,48) + loop*6, CALC(9,23), 6, head_light, head_dark, LPLANE, DPLANE);
	}
	for (loop = 0; loop < 24; loop++) {
		GraySprite8_OR_R(CALC(8,48) + loop*6, CALC(87,101), 6, head_light, head_dark, LPLANE, DPLANE);
	}
	for (loop = 0; loop < 12; loop++) {
		GraySprite8_OR_R(CALC(8,48), CALC(15,29) + loop*6, 6, head_light, head_dark, LPLANE, DPLANE);
	}
	for (loop = 0; loop < 12; loop++) {
		GraySprite8_OR_R(CALC(146,186), CALC(15,29) + loop*6, 6, head_light, head_dark, LPLANE, DPLANE);
	}
	
}

void EraseMenu(void) {
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(7,47), CALC(8,21), CALC(152,192), CALC(93,107), COLOR_WHITE);
	GraySpriteX8_OR(CALC(0,40), CALC(0,14), 100, plane_light, plane_dark, 20, LPLANE, DPLANE); 
	
}


/*void DrawSprites(void) {
	
	unsigned int loop1;
	
	
	for (loop1 = 0; loop1 < 14; loop1++) {
		
		GraySprite8_OR_R(4, 4 + loop1*6, 6, font[loop1], font[loop1], LPLANE, DPLANE);
		
	} */
	/*for (loop1 = 14; loop1 < 27; loop1++) {
		
		GraySprite8_OR_R(4 + (loop1-14)*6, 9, 6, font[loop1], font[loop1], LPLANE, DPLANE);
		
	}
	
	GraySprite8_OR_R(1, 17, 6, head_light, head_dark, LPLANE, DPLANE);
	
	for (loop1 = 0; loop1 < 8; loop1++) {
		
		GraySprite8_OR_R(7 + loop1*6, 17, 6, limb_light, limb_dark, LPLANE, DPLANE);
		
	}*/
	
//}