/******************************************************************************
*
* project name:    Bookworm
* file name:       extern##.h
* initial date:    3.07.2005
* author:          mhubi@freesurf.ch
* description:     all to do with extern data
*
*******************************************************************************
*
* Bookworm -- another snake game
* Copyright (C) 2005  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h10.h"


//get the pointer to a file (from Travis Fischer's Excitebike 68k)
unsigned char *GetDataPtr(const char *Filename, short Offset) {
	
	//variables
	unsigned char *Location;
	SYM_ENTRY *SymPtr;
	
	//search the file
	//tigcclib: vat.h
	if ((SymPtr = SymFindPtr(SYMSTR(Filename), 0)) == NULL) {
		//error-handling if not found
		return NULL;
	}
	
	//grab the address
	//tigcclib: alloc.h
	if ((Location = (unsigned char*) HeapDeref(SymPtr->handle)) == NULL) {
		//error-handling
		return NULL;
	}
	
	//return the pointer
	return (Location + 2 + Offset);
	
}


//archives a file (from Travis Fischer's Excitebike 68k)
void Archive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	//tigcclib: vat.h
	HSym hsym = SymFind(SYMSTR(Filename));
	
	//tigcclib: vat.h
	SymPtr = DerefSym (hsym);
	
	//look if there could be garbage collection
	//tigcclib: flash.h
	if (EM_findEmptySlot(*GetDataPtr(Filename, -2)) == NULL) {
		return;
	}
	//archive it (if not already archive)
	if (!SymPtr->flags.bits.archived) {
		
		//tigcclib: vat.h
		EM_moveSymToExtMem(SYMSTR(Filename), HS_NULL);
		
	}
		
}


//unarchives a file (from Travis Fischer's Excitebike 68k)
void Unarchive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	//tigcclib: vat.h
	HSym hsym = SymFind(SYMSTR(Filename));
	
	//tigcclib: vat.h
	SymPtr = DerefSym (hsym);
	
	//if it's archive unarchive it
	if (SymPtr->flags.bits.archived) {
		
		//tigcclib: vat.h
		EM_moveSymFromExtMem(SYMSTR(Filename), HS_NULL);
		
	}
	
}


//saves the hiscore table to the hiscore file
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int SaveHiscores(SCORETABLE *hiscores) {
	
	//variables
	unsigned int success = TRUE;
	FILE *f;
	
	//unarchive file
	//own function
	Unarchive(scorefile);
	
	//delete any old scorefile
	//tigcclib: stdio.h
	unlink(scorefile);

	//open the hiscore file
	//tigcclib: stdio.h
	if ((f = fopen(scorefile, "wb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//write the hiscore table to disk - check for errors
		//tigcclib: stdio.h
		if (fwrite(hiscores, sizeof(SCORETABLE), MAX_HISCORES, f) != MAX_HISCORES) {
			
			//error-handling
			success = FALSE;
			
		}

		//write the file tag
		//tigcclib: stdio.h
		fputc(0, f);
		fputs("BWHS", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//archive the file
	//own function
	Archive(scorefile);
	
	return success;
	
}


//loads the hiscores from file into the hiscore table
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int LoadHiscores(SCORETABLE *hiscores) {
	
	//variables
	unsigned int success = TRUE;
	FILE *f;

	//open file for reading and check for errors
	//tigcclib: stdio.h
	if ((f = fopen(scorefile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read the hiscore table and check for errors
		//tigcclib: stdio.h
		if (fread(hiscores, sizeof(SCORETABLE), MAX_HISCORES, f) != MAX_HISCORES) {
			
			//error-handling
			success = FALSE;
			
		}
		
		//close file
		//tigcclib: stdio.h
		fclose(f);
		
	}

	return success;
	
}


//create a blank hiscore table and write it to file
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int CreateHiscoreFile(void) {
	
	//variables
	unsigned int loop1;
	SCORETABLE board[MAX_HISCORES];

	//set all the entries in the table to default (defined in data##.h)
	for (loop1 = 0; loop1 < MAX_HISCORES; loop1++) {
		
		//copy the score
		board[loop1].score = hisc_score[loop1];
		
		//copy the name
		strcpy(board[loop1].name, hisc_name[loop1]);
		
	}
	
	//save it and return this value
	//changed function from www.technoplaza.net
	return (SaveHiscores(board));
	
}

//checks whether a hiscore file already exists
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int NeedHiscoreFile(void) {
	
	//variables
	unsigned int needed = FALSE;
	//tigcclib: stdio.h
	FILE *f = fopen(scorefile, "rb");
	
	//if we cannot open the file, we must announce it to the previous routine
	if (f == NULL) {
		needed = TRUE;
	}
	else {
		
		//close the file
		//changed function from www.technoplaza.net
		fclose(f);
		
	}

	return needed;
	
}


//update the hiscores
//prototype from www.technoplaza.net / C-Programming/Graduate Review (Nibbler-game)
void CalcHiscores(unsigned int newscore) {
	
	//variables
	SCORETABLE scores[MAX_HISCORES];
	int loop1;
	int pos = -1;
	char name[12];

	//look if there's a hiscore-file
	//changed function from www.technoplaza.net
	if (NeedHiscoreFile() == TRUE) {
		
		//if there isn't one, create one
		//changed function from www.technoplaza.net
		if (!CreateHiscoreFile()) {
			
			//error-handling
			//own function
			LeadOut(ERRORSAVE);
			
		}
		
	}
	
	//load the hiscore-file
	//changed function from www.technoplaza.net
	if (!LoadHiscores(scores)) {
		
		//error-handling
		//own function
		LeadOut(ERRORLOAD);
		
	}

	//check if latest score is a hiscore
	for (loop1 = (MAX_HISCORES - 1); loop1 >= 0; loop1--) {
		
		//if it is
		if (newscore > scores[loop1].score) {
			
			//save the position
			pos = loop1;
			
		}
		
	}
	
	//if there is a new
	if (pos != -1) {
		
		DrawMenuFrame();
		
		GrayDrawStrExt(0, CALC(25,39), "NEW HISCORE!", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		
		//get the name
		//own function
		Inputstr(name, 8);
		
		//EraseMenu();
		
		//update the whole hiscore-list (only if it's better than the last value
		//if (pos < (MAX_HISCORES - 1)) { //done before
			//move the necessary entries
			for (loop1 = (MAX_HISCORES - 1); loop1 > pos; loop1--) {
				
				//move the score
				scores[loop1].score = scores[loop1 - 1].score;
				//truncate the old name
				scores[loop1].name[0] = 0;
				//move the name
				strcpy(scores[loop1].name, scores[loop1 - 1].name);
				
			}
		//}

		// fill in the new hiscore
		scores[pos].score = newscore;
		scores[pos].name[0] = 0;
		strcpy(scores[pos].name, name);
		
		//save the table into the file
		//changed function from www.technoplaza.net
		if (!SaveHiscores(scores)) {
			
			//error handling
			//own function
			LeadOut(ERRORSAVE);
			
		}
		
	}
	else {
		
		return;
		
	}
	
	//DrawMenuFrame();
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(14,54), CALC(15,29), CALC(145,185), CALC(86,100), COLOR_WHITE);
	
	DrawHiscores(scores);
	
	EraseMenu();
	
	
	
}


//draws the hiscore-table
void DrawHiscores(SCORETABLE *scores) {
	
	//variables
	unsigned int loop1;
	int status_save;
	char string[16];
	
	status_save = game->status;
	game->status = MENU;
	
	//draw the title
	//extgraph.h
	GrayDrawStrExt2B(0, CALC(20,34), "HIGHSCORES", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10, LPLANE, DPLANE);
	
	//change the font
	//tigcclib: graph.h
	FontSetSys(F_6x8);
	
	//loop through the five entries
	for (loop1 = 0; loop1 < 5; loop1++) {
		
		//print the rank
		//tigcclib: stdio.h, string.h
		sprintf(string, "%d.", loop1 + 1);
		//extgraph.h
		GrayDrawStr2B(CALC(21,61), CALC(20,34) + 16 + loop1*9, string, A_NORMAL, DPLANE, DPLANE);
		//print the name
		//extgraph.h
		GrayDrawStr2B(CALC(36,76), CALC(20,34) + 16 + loop1*9, scores[loop1].name, A_NORMAL, DPLANE, DPLANE);
		//print the score
		//tigcclib: stdio.h, string.h
		sprintf(string, "%lu", scores[loop1].score);
		//extgraph.h
		GrayDrawStr2B(CALC(95,135), CALC(20,34) + 16 + loop1*9, string, A_NORMAL, DPLANE, DPLANE);
		
	}
	
	//wait on user-key
	//own function
	Waitkey();
	
	game->status = status_save;
	
	
}


void ShowHiscores(void) {
	
	SCORETABLE scores[MAX_HISCORES];
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(14,54), CALC(15,29), CALC(145,185), CALC(86,100), COLOR_WHITE);
	
	if (!(LoadHiscores(scores))) {
		GrayDrawStrExt2B(0, CALC(20,34), "error or no data", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10, LPLANE, DPLANE);
		Waitkey();
	}
	else {
		DrawHiscores(scores);
	}
	
	GrayFastFillRect_R(LPLANE, DPLANE, CALC(14,54), CALC(15,29), CALC(145,185), CALC(86,100), COLOR_WHITE);
	
}
