/******************************************************************************
*
* project name:    Bookworm
* file name:       main##.h
* initial date:    3.07.2005
* author:          mhubi@freesurf.ch
* description:     main-c-file
*
*******************************************************************************
*
* Bookworm -- another snake game
* Copyright (C) 2005  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h10.h"

#if defined(TI_BIG)
	#warning "Building the ti92+/v200 version"
#elif defined(TI_SMALL)
	#warning "Building the ti89 version"
#endif

#if defined(VTI)
	#warning "Building the VTI-version"
#endif

//global variables
INT_HANDLER save_int_1;
INT_HANDLER save_int_5;
char *homescreen;
void *bulk;
GAME *game;
WORM *worm;
char *plane_light;
char *plane_dark;
PMARK *pmark;


// Main Function
void _main(void)
{
	
	save_int_1 = GetIntVec (AUTO_INT_1);
	save_int_5 = GetIntVec (AUTO_INT_5);
	
	MemoryAllocation();
	
	game->status = NEWGAME;
	
	OptionsOpened = FALSE;
	
	LCD_save(homescreen);
	
	if (NeedHiscoreFile() == TRUE) {
		BoolsoftIntro(1);
	}
	
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);
	
	if(!GrayOn()) {
		LeadOut(MEMORY);
	}
	GrayClearScreen();
	
	randomize();
	
	game->level = 1;
	game->speed = 1;
	
	while (game->status == NEWGAME) {
			
		game->status = NORMAL;
		
		DrawWelcome();
		
		ChooseOptions();
		
		GameLoop();
		
	}
	
	LeadOut(QUIT);
	
}


void MemoryAllocation(void) {
	
	bulk = NULL;
	
	if ((bulk = malloc(BULKSIZE)) == NULL) {
		LeadOut(MEMORY);
	}
	
	homescreen = 	(char*)((char*)bulk);
	game =				(GAME*)((char*)homescreen + LCD_SIZE);
	worm =        (WORM*)((char*)game + sizeof(GAME));
	plane_light = (char*)((char*)worm + sizeof(WORM));
	plane_dark =  (char*)((char*)plane_light + LCD_SIZE);
	pmark =				(PMARK*)((char*)plane_dark + LCD_SIZE);
	
	
}


void Menu(void) {
	
	int status_save;
	int key;
	
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);
	status_save = game->status;
	game->status = MENU;
	
	DrawMenuFrame();
	
	do {
		
		GrayDrawStrExt(0, CALC(20,34), "bookworm "COMMENT_VERSION_STRING, A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt(0, CALC(38,52), "(1) Return  ", A_NORMAL | A_CENTERED, F_6x8);
		GrayDrawStrExt(0, CALC(47,61), "(2) Hiscores", A_NORMAL | A_CENTERED, F_6x8);
		GrayDrawStrExt(0, CALC(56,70), "(3) Exit    ", A_NORMAL | A_CENTERED, F_6x8);
		GrayDrawStrExt2B(0, CALC(72,86), "programmed by mhubi", A_NORMAL | A_CENTERED, F_4x6, DPLANE, DPLANE);
		GrayDrawStrExt2B(0, CALC(80,94), "-- boolsoft 05 --", A_NORMAL | A_CENTERED, F_4x6, DPLANE, DPLANE);
		
		do {
			
			while (!(key = KeyPressed())) {
				pokeIO(0x600005, 0b10111);
			}
			
		} while (key < K_1 || key > K_3);
		
		if (key == K_2) {
			ShowHiscores();
		}
		else if (key == K_3) {
			LeadOut(NORMAL);
		}
		
	} while (key != K_1);
	
	game->status = status_save;
	
	EraseMenu();
	
	if (OptionsOpened) {
		SetIntVec(AUTO_INT_5, CursorBlink);
	}
	
	Wait();
	
}


void LeadOut(int mode) {
	
	if (mode == MEMORY) {
		ST_helpMsg("Free up RAM (about 40'000 bytes)");
		free(bulk);
		exit(0);
	}
	
	Wait();
	
	GrayOff();
	
	SetIntVec (AUTO_INT_1, save_int_1);
	SetIntVec (AUTO_INT_5, save_int_5);
	
	LCD_restore(homescreen);
	
	if (mode == ERRORSAVE) {
		ST_helpMsg("ERROR: saving hiscores");
	}
	else if (mode == ERRORLOAD) {
		ST_helpMsg("ERROR: loading hiscores");
	}
	else if (mode != FASTQUIT) {
		ST_helpMsg(CALC("~~ bookworm "COMMENT_VERSION_STRING"   by boolsoft ~~",
										"                           ~~ bookworm "COMMENT_VERSION_STRING"   by boolsoft ~~"));
	}
	
	free(bulk);
	
	exit(0);
	
}

void Delay(unsigned int delay, unsigned int mode) {
	
	unsigned int loop;
	
	for (loop = 0; loop <= delay; loop++) {
		#if defined(VTI)
		unsigned int loop_vti;
		for (loop_vti = 0; loop_vti < 1430; loop_vti++);
		#else
		if (mode == 0) {
			pokeIO(0x600005, 0b10111);
		}
		else if (mode == 1) {
			pokeIO(0x600005, 0b10101);
		}
		#endif
		
	}
	
}